@@ -110,10 +110,10 @@ gst_rtp_vorbis_pay_class_init (GstRtpVorbisPayClass * klass)
   gobject_class->set_property = gst_rtp_vorbis_pay_set_property;
   gobject_class->get_property = gst_rtp_vorbis_pay_get_property;
 
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&gst_rtp_vorbis_pay_src_template));
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&gst_rtp_vorbis_pay_sink_template));
+  gst_element_class_add_static_pad_template (gstelement_class,
+      &gst_rtp_vorbis_pay_src_template);
+  gst_element_class_add_static_pad_template (gstelement_class,
+      &gst_rtp_vorbis_pay_sink_template);
 
   gst_element_class_set_static_metadata (gstelement_class,
       "RTP Vorbis payloader",
@@ -145,23 +145,18 @@ gst_rtp_vorbis_pay_clear_packet (GstRtpVorbisPay * rtpvorbispay)
   if (rtpvorbispay->packet)
     gst_buffer_unref (rtpvorbispay->packet);
   rtpvorbispay->packet = NULL;
-  g_list_foreach (rtpvorbispay->packet_buffers, (GFunc) gst_mini_object_unref,
-      NULL);
-  g_list_free (rtpvorbispay->packet_buffers);
+  g_list_free_full (rtpvorbispay->packet_buffers,
+      (GDestroyNotify) gst_buffer_unref);
   rtpvorbispay->packet_buffers = NULL;
 }
 
 static void
 gst_rtp_vorbis_pay_cleanup (GstRtpVorbisPay * rtpvorbispay)
 {
-  g_list_foreach (rtpvorbispay->headers, (GFunc) gst_mini_object_unref, NULL);
-  g_list_free (rtpvorbispay->headers);
-  rtpvorbispay->headers = NULL;
-
   gst_rtp_vorbis_pay_clear_packet (rtpvorbispay);
-
-  if (rtpvorbispay->config_data)
-    g_free (rtpvorbispay->config_data);
+  g_list_free_full (rtpvorbispay->headers, (GDestroyNotify) gst_buffer_unref);
+  rtpvorbispay->headers = NULL;
+  g_free (rtpvorbispay->config_data);
   rtpvorbispay->config_data = NULL;
   rtpvorbispay->last_config = GST_CLOCK_TIME_NONE;
 }
@@ -277,12 +272,7 @@ gst_rtp_vorbis_pay_init_packet (GstRtpVorbisPay * rtpvorbispay, guint8 VDT,
 {
   GST_LOG_OBJECT (rtpvorbispay, "starting new packet, VDT: %d", VDT);
 
-  if (rtpvorbispay->packet)
-    gst_buffer_unref (rtpvorbispay->packet);
-  g_list_foreach (rtpvorbispay->packet_buffers, (GFunc) gst_mini_object_unref,
-      NULL);
-  g_list_free (rtpvorbispay->packet_buffers);
-  rtpvorbispay->packet_buffers = NULL;
+  gst_rtp_vorbis_pay_clear_packet (rtpvorbispay);
 
   /* new packet allocate max packet size */
   rtpvorbispay->packet =
@@ -517,8 +507,7 @@ gst_rtp_vorbis_pay_finish_headers (GstRTPBasePayload * basepayload)
   configuration = g_base64_encode (config, configlen);
 
   /* store for later re-sending */
-  if (rtpvorbispay->config_data)
-    g_free (rtpvorbispay->config_data);
+  g_free (rtpvorbispay->config_data);
   rtpvorbispay->config_size = configlen - 4 - 3 - 2;
   rtpvorbispay->config_data = g_malloc (rtpvorbispay->config_size);
   rtpvorbispay->config_extra_len = extralen;
@@ -684,7 +673,7 @@ gst_rtp_vorbis_pay_payload_buffer (GstRtpVorbisPay * rtpvorbispay, guint8 VDT,
       for (l = rtpvorbispay->headers; l; l = l->next)
         rtpvorbispay->packet_buffers =
             g_list_prepend (rtpvorbispay->packet_buffers,
-            gst_buffer_ref (buffer));
+            gst_buffer_ref (l->data));
     }
 
     /* only first (only) configuration cuts length field */
