@@ -103,6 +103,7 @@ struct _GstRTSPStream {
   gboolean      skipped;
   gboolean      eos;
   gboolean      discont;
+  gboolean      need_caps;
 
   /* for interleaved mode */
   guint8        channel[2];
@@ -242,6 +243,7 @@ struct _GstRTSPSrc {
   gint              ntp_time_source;
   gchar            *user_agent;
   GstClockTime      max_rtcp_rtp_time_diff;
+  gboolean          rfc7273_sync;
 
   /* state */
   GstRTSPState       state;
